//
//  Pushbots.m
//  Pushbots framework 2.0.4
//
//  Created by Abdullah Diaa on 06/09/16.
//  Copyright (c) 2016 PushBots Inc. All rights reserved.
//

#import "Pushbots.h"
#import "DeviceInfo.h"

#import <stdlib.h>
#import <stdio.h>
#import <sys/types.h>
#import <sys/sysctl.h>
#import <UIKit/UIKit.h>


//static NSString *const PUSHBOTS_API = @"http://192.168.1.100:4000";


static NSString *const PUSHBOTS_API = @"https://api.pushbots.com";
static NSString *const PUSHBOTS_SDK_VERSION = @"2.0.4";
static PBLogLevel _pbloglevel = PBLogLevelInfo;

//Private methods/properries
@interface Pushbots ()

@property (nonatomic, strong)NSOperationQueue *operationQueue;

+ (Pushbots *)sharedInstance;
+ (void)setLogLevel:(PBLogLevel)pbloglevel;
- (void)setObjectId:(NSString *)objectId;
- (NSString *)objectId;
- (void)HTTPRequest:(NSString *) verb url:(NSString *)url params:(NSDictionary *)params completionHandler:(void (^)(NSHTTPURLResponse* response, NSData* data, NSError* connectionError)) handler;
- (void)HTTPRequest:(NSString *) verb url:(NSString *)url completionHandler:(void (^)(NSHTTPURLResponse* response, NSData* data, NSError* connectionError)) handler;
- (void)HTTPRequest:(NSString *) verb url:(NSString *)url params:(NSDictionary *)params;
@end

@implementation Pushbots
static Pushbots *sharedInstance = nil;

NSString* deviceToken;
bool subscriptionStatus = true;
bool registeredAPNS = false;
int registeredNotificationTypes = -1;

@synthesize applicationId = _applicationId;
@synthesize deviceId = _deviceId;
@synthesize objectId = _objectId;
@synthesize operationQueue = _operationQueue;
@synthesize prompt = _prompt;

#pragma mark - Initilize methods

- (id)initWithAppId:(NSString*)appId{
    return [self initWithAppId:appId prompt:true];
}

- (id)initWithAppId:(NSString*)appId prompt:(BOOL)prompt  {
    self = [self init];
    if (self) {
        //Queue for requests
        _operationQueue = [[NSOperationQueue alloc] init];
        
        //Store parameters
        self.applicationId = appId;
        self.prompt = prompt;
        
        //Store sharedInstance
        if (sharedInstance == nil)
            sharedInstance = self;
        
        NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
        deviceToken = [defaults objectForKey:@"com.pushbots.api.deviceID"];

        //currentUserNotificationSettings in iOS 8 ONLY
        if (([[UIApplication sharedApplication] respondsToSelector:@selector(currentUserNotificationSettings)])){
            registeredAPNS = [[UIApplication sharedApplication] currentUserNotificationSettings].types != (NSUInteger)nil;
        }else{
            //Check token value in standardUserDefaults with pre-iOS 8
            registeredAPNS = deviceToken != nil || [defaults boolForKey:@"com.pushbots.api.deviceID.registered"];
        }
        
        //Set subscriptionStatus locally, if not stored at all then it's nil or TRUE
        subscriptionStatus = [defaults objectForKey:@"com.pushbots.subscription"] == nil;
        
        registeredNotificationTypes = getNotificationTypes();

        if(registeredAPNS || self.prompt){
            [self registerForRemoteNotifications];
        }
    }
    return self;
}

+ (Pushbots*)sharedInstance {
    return sharedInstance;
}

+ (void)setLogLevel:(PBLogLevel)pbloglevel {
    _pbloglevel = pbloglevel;
}

#pragma mark - Remote Notifications reg.

- (void)registerForRemoteNotifications {
#ifdef __IPHONE_8_0
    if ([[UIApplication sharedApplication] respondsToSelector:@selector(registerUserNotificationSettings:)]) {
        UIUserNotificationSettings* notificationSettings = [UIUserNotificationSettings settingsForTypes:(UIUserNotificationTypeAlert | UIUserNotificationTypeBadge | UIUserNotificationTypeSound) categories:nil];
        [[UIApplication sharedApplication] registerUserNotificationSettings:notificationSettings];
        [[UIApplication sharedApplication] registerForRemoteNotifications];
    } else {
        [[UIApplication sharedApplication] registerForRemoteNotificationTypes: (UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeSound | UIRemoteNotificationTypeAlert)];
    }
#else
    // Register device for iOS7
    [[UIApplication sharedApplication] registerForRemoteNotificationTypes: (UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeSound | UIRemoteNotificationTypeAlert)];
    if(!registeredAPNS){
        NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
        [defaults setObject:deviceID forKey:@"com.pushbots.api.deviceID.registered"];
        [defaults synchronize];
    }
    
#endif
}

- (void) registerOnPushbots:(NSData *)deviceToken {
    //Device Token
    //Remove < , > and spaces to make it in binary format for performance
    NSString *deviceID = [[[deviceToken description] stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"<>"]] stringByReplacingOccurrencesOfString:@" " withString:@""];
    self.deviceId = deviceID;
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:deviceID forKey:@"com.pushbots.api.deviceID"];
    [defaults synchronize];
    [self subscribe];
}

- (void) getDevice:(void (^)(NSDictionary *device, NSError *error))callback {
    
    if (![self objectId] || [[self objectId] isEqualToString:@""])
        return;
    
    NSString *url = [NSString stringWithFormat:@"%@/2/subscriptions/%@", PUSHBOTS_API, [self objectId]];
    
    [self HTTPRequest:@"GET"
                  url:url
    completionHandler:^(NSHTTPURLResponse *response, NSData *data, NSError *connectionError) {
        if(connectionError) {
            return callback(nil, connectionError);
        }
        
        NSError *error;
        NSDictionary *device = [NSJSONSerialization JSONObjectWithData:data options:kNilOptions error:&error];
        return callback(device, error);
    }];
}

-(void) subscribe {
    //Request parameters
    NSMutableDictionary *params = [NSMutableDictionary dictionary];
    NSString *token = [[NSUserDefaults standardUserDefaults] objectForKey:@"com.pushbots.api.deviceID"];
    
    if (!token || [token isEqualToString:@""])
        return;

    //@Required
    [params setObject:token forKey:@"token"];
    // UUID to fix duplicate notificaitons with iOS9+
    [params setObject:[[[UIDevice currentDevice] identifierForVendor] UUIDString] forKey:@"udid"];
    [params setObject:@"0" forKey:@"platform"];
    
    //@Optional
    if([DeviceInfo timezone])
        [params setObject:[DeviceInfo timezone] forKey:@"timeZone"];
    
    if([DeviceInfo releaseMode] != nil)
        [params setObject:[DeviceInfo releaseMode] forKey:@"releaseMode"];
    
    if([DeviceInfo country])
        [params setObject:[DeviceInfo country] forKey:@"countryCode"];
    
    //@Device Metrics
    NSMutableDictionary *metrics = [NSMutableDictionary dictionary];
    if([DeviceInfo appVersion])
        [metrics setObject:[DeviceInfo appVersion]  forKey:@"appVersion"];
    
    if([DeviceInfo resolution])
        [metrics setObject:[DeviceInfo resolution] forKey:@"resolution"];
    
    [metrics setObject:[DeviceInfo device] forKey:@"deviceModel"];
    
    if([DeviceInfo locale])
        [metrics setObject:[DeviceInfo locale] forKey:@"locale"];
    [metrics setObject:PUSHBOTS_SDK_VERSION forKey:@"PushBotslibVersion"];
    [metrics setObject:[DeviceInfo osVersion] forKey:@"osVersion"];
    [metrics setObject:[NSNumber numberWithBool:[DeviceInfo isJailbroken]] forKey:@"jailbroken"];
    
    if([DeviceInfo carrier])
        [metrics setObject:[DeviceInfo carrier] forKey:@"carrier"];
    
    [params setObject:metrics forKey:@"metrics"];
    
    NSString *url = [NSString stringWithFormat:@"%@/2/subscriptions", PUSHBOTS_API];
    
    [self HTTPRequest:@"POST"
                  url:url
               params:params];
}


-(void) update:(NSDictionary *)deviceObject {
    
    if (![self objectId] || [[self objectId] isEqualToString:@""])
        return;
    
    NSString *url = [NSString stringWithFormat:@"%@/2/subscriptions/%@", PUSHBOTS_API, [self objectId]];
    
    [self HTTPRequest:@"PUT"
                  url:url
               params:deviceObject];
}

- (void) toggleNotifications:(BOOL)subscribed{
    NSDictionary *params = @{@"subscribed" : @(subscribed)};
    [self update:params];
}

- (void) debug:(BOOL)debug{
    NSDictionary *params = @{@"debug" : @(debug)};
    [self update:params];
}

- (void) setAlias:(NSString *)alias{
    NSDictionary *params = @{@"alias" : alias};
    [self update:params];
}

- (void) removeAlias;{
    NSDictionary *params = @{@"remove_alias" : @"true"};
    [self update:params];
}

#pragma mark - Handle tags

- (void) tag:(NSArray *)tags{
    NSDictionary *params = @{@"tags_add" : tags};
    [self update:params];
}

- (void) untag:(NSArray *)tags{
    NSDictionary *params = @{@"tags_remove" : tags};
    [self update:params];
}


#pragma mark - Handle device badge

- (void) setBadge: (int) number{
    [UIApplication sharedApplication].applicationIconBadgeNumber = number;
    NSDictionary *params = @{@"badge" : @(number)};
    [self update:params];
}

- (void) clearBadgeCount {
    clearLocalBadgeCount(false);
    NSDictionary *params = @{@"badge" : @0};
    [self update:params];
}

- (void) clearBadgeCountfromNotification: (BOOL)fromNotification {
    clearLocalBadgeCount(fromNotification);
    NSDictionary *params = @{@"badge" : @0};
    [self update:params];
}
- (void) decrementBadgeCountBy: (int) number {
    [UIApplication sharedApplication].applicationIconBadgeNumber = [UIApplication sharedApplication].applicationIconBadgeNumber - number;
    NSDictionary *params = @{@"badge_d" : @(number)};
    [self update:params];
}

- (void) incrementBadgeCountBy: (int) number {
    [UIApplication sharedApplication].applicationIconBadgeNumber = [UIApplication sharedApplication].applicationIconBadgeNumber - number;
    NSDictionary *params = @{@"badge_p" : @(number)};
    [self update:params];
}

+ (void)openURL:(NSDictionary *)userInfo{
    //Try to get URL from [didReceiveRemoteNotification] dictionary
    NSString* openURL = [userInfo objectForKey:@"openURL"];
    
    if (openURL != nil) {
        NSURL* valid_url = [NSURL URLWithString:openURL];
        if (valid_url){
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.5 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
                [[UIApplication sharedApplication] openURL:valid_url];
            });
        }
    }
}

bool clearLocalBadgeCount(bool fromNotification) {
    if (registeredNotificationTypes == -1 || (registeredNotificationTypes & 1) == 0)
        return false;
    
    bool badgeMoreThanZero = [UIApplication sharedApplication].applicationIconBadgeNumber > 0;

    if ((!(NSFoundationVersionNumber > NSFoundationVersionNumber_iOS_7_1) && fromNotification) || badgeMoreThanZero) {
        [[UIApplication sharedApplication] setApplicationIconBadgeNumber:1];
        [[UIApplication sharedApplication] setApplicationIconBadgeNumber:0];
    }
    return badgeMoreThanZero;
}

int getNotificationTypes() {
    if (!subscriptionStatus)
        return -2;
    
    if (deviceToken) {
        if ([[UIApplication sharedApplication] respondsToSelector:@selector(currentUserNotificationSettings)])
            return [[UIApplication sharedApplication] currentUserNotificationSettings].types;
        else
            return 7;
    }
    
    return -1;
}

#pragma mark - Track notifications open

-(void) trackPushNotificationOpenedWithLaunchOptions:(NSDictionary *) launchOptions{
    
    NSDictionary* payload = [launchOptions objectForKey:UIApplicationLaunchOptionsRemoteNotificationKey];
    
    if(payload){
        [self clearBadgeCountfromNotification:true];
        [[UIApplication sharedApplication] cancelAllLocalNotifications];
        NSString* push_id = [payload valueForKey:@"pb_n_id"];
        
        if(push_id){
            [self PushNotificationOpenedWithId:push_id];
        }
    }
}

-(void) trackPushNotificationOpenedWithPayload:(NSDictionary *) payload{
    [self clearBadgeCountfromNotification:true];
    [[UIApplication sharedApplication] cancelAllLocalNotifications];
    NSString* push_id = [payload valueForKey:@"pb_n_id"];
    
    if(push_id){
        [self PushNotificationOpenedWithId:push_id];
    }
}

-(void) PushNotificationOpenedWithId:(NSString *) push_id{
    NSMutableDictionary *params = [NSMutableDictionary dictionary];
    
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    NSLocale *enUSPOSIXLocale = [NSLocale localeWithLocaleIdentifier:@"en_US_POSIX"];
    [dateFormatter setLocale:enUSPOSIXLocale];
    [dateFormatter setDateFormat:@"yyyy-MM-dd'T'HH:mm:ssZZZZZ"];
    
    NSDate *now = [NSDate date];
    NSString *iso8601String = [dateFormatter stringFromDate:now];
    
    //@Required
    [params setObject:push_id forKey:@"push_id"];
    [params setObject:self.objectId forKey:@"device_id"];
    [params setObject:iso8601String forKey:@"date"];
    [params setObject:@"0" forKey:@"platform"];

    NSString *url = [NSString stringWithFormat:@"%@/2/pushOpened", PUSHBOTS_API];
    
    [self HTTPRequest:@"POST"
                  url:url
               params:params];
}

#pragma mark - logging
void PushbotsLog(PBLogLevel logLevel, NSString* message) {
    NSString* levelString;
    switch (logLevel) {
        case PBLogLevelVerbose:
            levelString = @"PUSHBOTS VERBOSE: ";
            break;
        case PBLogLevelInfo:
            levelString = @"PUSHBOTS INFO: ";
            break;
        case PBLogLevelWarn:
            levelString = @"PUSHBOTS WARNING: ";
            break;
        case PBLogLevelError:
            levelString = @"PUSHBOTS ERROR: ";
            break;
        default:
            break;
    }
    
    if (logLevel <= _pbloglevel)
        NSLog(@"%@", [levelString stringByAppendingString:message]);
}

#pragma mark - Handle objectId

- (BOOL)subscriptionStatus
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    return [defaults boolForKey:@"com.pushbots.subscription"];
}

- (void)setSubscriptionStatus:(BOOL)status
{
    NSString* value = nil;
    if (!status)
        value = @"no";
    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:value forKey:@"com.pushbots.subscription"];
    [defaults synchronize];
    subscriptionStatus = status;
}



- (NSString *)objectId
{
    if (_objectId == nil) {
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        _objectId = [defaults stringForKey:@"com.pushbots.api.object_id"];
    }
    
    if (_objectId == nil) {
        return @"";
    }
    
    return _objectId;
}

- (void)setObjectId:(NSString *)objectId
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    if (objectId != nil) {
        [defaults setObject:objectId forKey:@"com.pushbots.api.object_id"];
        [defaults synchronize];
    }else{
        [defaults removeObjectForKey:@"com.pushbots.api.object_id"];
    }
    _objectId = objectId;
}

#pragma mark - HTTP Requests

- (NSDictionary *)userInfoForData:(id)data andResponse:(NSHTTPURLResponse *)response
{
    NSMutableDictionary *userInfo = [NSMutableDictionary dictionary];
    if (data) {
        id json = [NSJSONSerialization JSONObjectWithData:data options:0 error:nil];
        json = json == nil ? [NSNull null] : json;
        [userInfo setObject:json forKey:@"response_data"];
    }
    if (response) {
        [userInfo setObject:response forKey:@"response"];
    }
    // make sure not to return a mutable dictionary
    return [NSDictionary dictionaryWithDictionary:userInfo];
}


-(void)HTTPRequest:(NSString *) verb url:(NSString *)url params:(NSDictionary *)params completionHandler:(void (^)(NSHTTPURLResponse* response, NSData* data, NSError* connectionError)) handler
{
    
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:url]];
    request.HTTPMethod = verb;
    
    if (params != nil)
    {
        NSError *jsonError;
        NSData *json = [NSJSONSerialization dataWithJSONObject:params options:0 error:&jsonError];
        
        if(jsonError) {
            return handler(nil, nil, jsonError);
        }

        request.HTTPBody = json;
    }
    
    [request setValue:self.applicationId forHTTPHeaderField:@"X-PUSHBOTS-APPID"];
    [request setValue:@"application/json; charset=utf-8" forHTTPHeaderField:@"Content-Type"];
    
    
    NSURLSessionTask *task = [[NSURLSession sharedSession] dataTaskWithRequest:request completionHandler:
                              ^(NSData *data, NSURLResponse *urlResponse, NSError *error) {
                                  if (data) {
                                      NSHTTPURLResponse *httpResponse = (NSHTTPURLResponse *) urlResponse;
                                      handler(httpResponse, data, error);
                                  } else {
                                      PushbotsLog(PBLogLevelError, [NSString stringWithFormat:@"Failed to fetch %@: %@", url, error ]);
                                  }
                              }];
    [task resume];
}

-(void)HTTPRequest:(NSString *)verb url:(NSString *)url completionHandler:(void (^)(NSHTTPURLResponse *, NSData *, NSError *))handler
{
    [self HTTPRequest:verb url:url params:nil completionHandler:handler];
}

-(void)HTTPRequest:(NSString *)verb url:(NSString *)url params:(NSDictionary *)params;
{
    [self HTTPRequest:verb url:url params:params completionHandler:^(NSHTTPURLResponse *response, NSData *data, NSError *error) {
        
        PushbotsLog(PBLogLevelInfo, [NSString stringWithFormat:@"%@ %@", verb, url ]);
        PushbotsLog(PBLogLevelInfo, [NSString stringWithFormat:@"Params: %@", [params description] ]);

        if (error) {
            PushbotsLog(PBLogLevelError, [NSString stringWithFormat:@"Request Error: %@", [error description] ]);
            return;
        }
        
        //Status code response
        NSInteger statusCode = [response statusCode];
        PushbotsLog(PBLogLevelInfo, [NSString stringWithFormat:@"Response status code: %ld", (long)statusCode]);

        
        NSError *jsonError = nil;
        NSJSONSerialization* result = [NSJSONSerialization JSONObjectWithData:data
                                                                      options:0
                                                                        error:&jsonError];
        if([result description])
            PushbotsLog(PBLogLevelInfo, [NSString stringWithFormat:@"Response: %@", [result description]]);
        
        //Handle response code
        
        if([url containsString:@"/2/pushOpened"] && [verb isEqualToString:@"POST"]){
            if(statusCode == 200){
                PushbotsLog(PBLogLevelInfo, @"Push analytics updated.");
            }
        }
        
        // @GET: /2/subscriptions/{id}
        if([url containsString:@"/2/subscriptions"] && [verb isEqualToString:@"GET"]){
            if(statusCode == 200){
                PushbotsLog(PBLogLevelInfo, [NSString stringWithFormat:@"Device data: %@", [result description]]);
            }
        }
        
        // @POST: /2/subscriptions
        if([url containsString:@"/2/subscriptions"] && [verb isEqualToString:@"POST"]){
            //Store ObjectId on device creation
            if(statusCode == 201){
                [Pushbots.sharedInstance setObjectId:[result valueForKey:@"_id"]];
                PushbotsLog(PBLogLevelInfo, [NSString stringWithFormat:@"Stored ObjectId %@", [Pushbots.sharedInstance objectId]]);
            }
            
            if(statusCode == 200){
                PushbotsLog(PBLogLevelInfo, [NSString stringWithFormat:@"Updating ObjectId %@", [result valueForKey:@"_id"]]);
                [Pushbots.sharedInstance setObjectId:[result valueForKey:@"_id"]];
            }
        }
        
        // @PUT: /2/subscriptions
        if([url containsString:@"/2/subscriptions"] && [verb isEqualToString:@"PUT"]){
            if(statusCode == 200){
                //Only if request succeed with subscribed in params, update it locally.
                if([params valueForKey:@"subscribed"]){
                    [self setSubscriptionStatus:[[params valueForKey:@"subscribed"] boolValue]];
                }
                PushbotsLog(PBLogLevelInfo, @"Updated Device data");
            }else if(statusCode == 202){
                //Reset objectId
                [Pushbots.sharedInstance setObjectId:nil];
                [[NSUserDefaults standardUserDefaults] removeObjectForKey:@"com.pushbots.api.deviceID"];
            }
        }
        
        // @DELETE: /2/subscriptions/{id}
        if([url containsString:@"/2/subscriptions"] && [verb isEqualToString:@"DELETE"]){
            if(statusCode == 200){
                if([result valueForKey:@"deleted"]){
                    [Pushbots.sharedInstance setObjectId:nil];
                    [[NSUserDefaults standardUserDefaults] removeObjectForKey:@"com.pushbots.api.deviceID"];
                }else{
                    PushbotsLog(PBLogLevelError, @"Couldn't delete subscription!");
                }
            }
        }
        
        if (statusCode >= 400) {
            PushbotsLog(PBLogLevelError, [NSString stringWithFormat:@"Response status code: %ld", (long)statusCode]);
        }
    }];
}
@end
